"""Top-level package for numpy_nn."""

__author__ = """Rahul D Shetty"""
__email__ = '35rahuldshetty@gmail.com'
__version__ = '0.1.0'

# Layers
from .layers import Neuron

# Losses
from .losses import MeanAbsoluteError
